@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.99 2003/02/26 10:00:01 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.101 2003/07/10 09:38:47 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -308,6 +308,10 @@ void CmdExec::RemoveFeeder()
    if(!feeder)
       return;
 
+   // save old cwd if necessary
+   if(interactive && feeder->prev==0)
+      cwd_history.Set(session,session->GetCwd());
+
    CmdFeeder *tmp=feeder->prev;
    next_cmd=cmd_buf=feeder->saved_buf;
    partial_cmd=false;
@@ -316,9 +320,6 @@ void CmdExec::RemoveFeeder()
    delete feeder;
    feeder=tmp;
 
-   // save old cwd if necessary
-   if(interactive && feeder==0)
-      cwd_history.Set(session,session->GetCwd());
 }
 
 void CmdExec::ReuseSavedSession()
@@ -474,11 +475,11 @@ int CmdExec::Do()
 	 }
 	 if(glob->Done() || glob->Error())
 	 {
-	    delete glob;
-	    glob=0;
 	    const char *pat=args->getnext();
 	    if(!pat)
 	    {
+	       delete glob;
+	       glob=0;
 	       // it was last argument
 	       delete args;
 	       args=args_glob;
@@ -491,7 +492,7 @@ int CmdExec::Do()
 	       exec_parsed_command();
 	       return MOVED;
 	    }
-	    glob=new GlobURL(session,pat);
+	    glob->NewGlob(pat);
 	    m=MOVED;
 	 }
 	 break;
