@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.46 2002/04/18 10:37:20 lav Exp $ */
+/* $Id: CmdExec.h,v 1.49 2002/07/31 11:27:59 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -124,6 +124,7 @@ private:
 
    char *old_cwd;
    char *old_lcwd;
+   char *slot;
 
    GlobURL *glob;
    ArgV *args_glob;
@@ -133,8 +134,9 @@ private:
    static CmdExec *chain;
    CmdExec *next;
 
-   QueueFeeder *has_queue;
+   QueueFeeder *queue_feeder;
    CmdExec *GetQueue(bool create = true);
+   bool SameQueueParameters(CmdExec *);
 
    FileAccess *saved_session;
    void ReuseSavedSession();
@@ -163,7 +165,7 @@ public:
    int Done();
    int ExitCode() { return exit_code; }
    int Do();
-   void PrintStatus(int);
+   void PrintStatus(int,const char *prefix="\t");
    void ShowRunStatus(StatusLine *s);
    int AcceptSig(int sig);
 
@@ -180,7 +182,6 @@ public:
    friend char *command_generator(char *,int);	  // readline completor
    static const char *GetFullCommandName(const char *);
 
-   char	 *var_ls;
    bool	 remote_completion;
    int	 long_running;
    bool	 csh_history;
@@ -239,6 +240,8 @@ public:
    bool	 wait_all;
 
    void pre_stdout();
+
+   void ChangeSlot(const char *n);
 };
 
 extern const char * const bookmark_subcmd[];
