@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -248,6 +248,7 @@ int Fish::Do()
       {
 	 state=FILE_SEND;
 	 real_pos=0;
+	 pos=0;
 	 m=MOVED;
       }
       if(RespQueueSize()==0)
@@ -260,7 +261,7 @@ int Fish::Do()
       }
       goto usual_return;
    case FILE_RECV:
-      if(recv_buf->Size()>=rate_limit->BytesAllowed())
+      if(recv_buf->Size()>=rate_limit->BytesAllowedToGet())
       {
 	 recv_buf->Suspend();
 	 Timeout(1000);
@@ -496,7 +497,7 @@ void Fish::SendArrayInfoRequests()
 
 void Fish::SendMethod()
 {
-   const char *e=shell_encode(file);
+   const char *e=alloca_strdup(shell_encode(file));
    const char *e1=shell_encode(file1);
    switch((open_mode)mode)
    {
@@ -510,7 +511,7 @@ void Fish::SendMethod()
       if(!encode_file)
 	 e=file;
       Send("#LIST %s\n"
-	   "ls -l %s; echo '### 200'\n",e,e);
+	   "ls -la %s; echo '### 200'\n",e,e);
       PushExpect(EXPECT_DIR);
       real_pos=0;
       break;
@@ -518,7 +519,7 @@ void Fish::SendMethod()
       if(!encode_file)
 	 e=file;
       Send("#LIST %s\n"
-	   "ls %s; echo '### 200'\n",e,e);
+	   "ls -a %s; echo '### 200'\n",e,e);
       PushExpect(EXPECT_DIR);
       real_pos=0;
       break;
@@ -538,17 +539,27 @@ void Fish::SendMethod()
       }
       // dd pays attansion to read boundaries and reads wrong number
       // of bytes when ibs>1. Have to use the inefficient ibs=1.
-      Send("#STOR %lld %s\n"
-           ">%s;echo '### 001';"
-	   "dd ibs=1 count=%lld 2>/dev/null"
-	   "|(cat>%s;cat>/dev/null);echo '### 200'\n",
-	   (long long)entity_size,e,
-	   e,
-	   (long long)entity_size,
-	   e);
+      if(entity_size>0)
+      {
+	 Send("#STOR %lld %s\n"
+	      ">%s;echo '### 001';"
+	      "dd ibs=1 count=%lld 2>/dev/null"
+	      "|(cat>%s;cat>/dev/null);echo '### 200'\n",
+	      (long long)entity_size,e,
+	      e,
+	      (long long)entity_size,
+	      e);
+      }
+      else
+      {
+	 Send("#STOR %lld %s\n"
+	      ">%s;echo '### 001';echo '### 200'\n",
+	      (long long)entity_size,e,e);
+      }
       PushExpect(EXPECT_STOR_PRELIMINARY);
       PushExpect(EXPECT_STOR);
       real_pos=0;
+      pos=0;
       break;
    case ARRAY_INFO:
       SendArrayInfoRequests();
@@ -632,7 +643,8 @@ int Fish::HandleReplies()
 	 const char *y="(yes/no)? ";
 	 int p_len=strlen(p);
 	 int y_len=strlen(y);
-	 if(s>=p_len && !strncasecmp(b+s-p_len,p,p_len))
+	 if(s>=p_len && !strncasecmp(b+s-p_len,p,p_len)
+	 || s>10 && !strncmp(b+s-3,"': ",3))
 	 {
 	    if(!pass)
 	    {
@@ -943,7 +955,7 @@ int Fish::Read(void *buf,int size)
 	 }
       }
 
-      int bytes_allowed=rate_limit->BytesAllowed();
+      int bytes_allowed=rate_limit->BytesAllowedToGet();
       if(size1>bytes_allowed)
 	 size1=bytes_allowed;
       if(size1==0)
@@ -965,7 +977,7 @@ int Fish::Read(void *buf,int size)
       recv_buf->Skip(size);
       pos+=size;
       real_pos+=size;
-      rate_limit->BytesUsed(size);
+      rate_limit->BytesGot(size);
       retries=0;
       return size;
    }
@@ -986,7 +998,7 @@ int Fish::Write(const void *buf,int size)
       return DO_AGAIN;
 
    {
-      int allowed=rate_limit->BytesAllowed();
+      int allowed=rate_limit->BytesAllowedToPut();
       if(allowed==0)
 	 return DO_AGAIN;
       if(size+send_buf->Size()>allowed)
@@ -1000,7 +1012,7 @@ int Fish::Write(const void *buf,int size)
       return 0;
    send_buf->Put((char*)buf,size);
    retries=0;
-   rate_limit->BytesUsed(size);
+   rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
    return(size);
@@ -1015,6 +1027,8 @@ int Fish::StoreStatus()
 {
    if(Error())
       return error_code;
+   if(state!=FILE_SEND)
+      return IN_PROGRESS;
    if(real_pos!=entity_size)
    {
       Disconnect();
