@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.156 2002/12/25 13:44:31 lav Exp $ */
+/* $Id: Http.cc,v 1.160 2003/06/21 08:40:06 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -298,6 +298,16 @@ void Http::SendMethod(const char *method,const char *efile)
       efile=pfile;
    }
 
+   /*
+      Handle the case when the user has not given us
+      get http://foobar.org (note the absense of the trailing /
+
+      It fixes segfault with a certain webserver which I've
+      seen ... (Geoffrey Lee <glee@gnupilgrims.org>).
+   */
+   if(*efile=='\0')
+      efile="/";
+
    Send("%s %s HTTP/1.1\r\n",method,efile);
    Send("Host: %s\r\n",ehost);
    if(user_agent && user_agent[0])
@@ -561,7 +571,7 @@ void Http::SendRequest(const char *connection,const char *f)
 void Http::SendArrayInfoRequest()
 {
    int m=1;
-   if(keep_alive)
+   if(keep_alive && use_head)
    {
       m=keep_alive_max;
       if(m==-1)
@@ -1115,7 +1125,7 @@ int Http::Do()
 		     return MOVED;
 		  }
 		  // we can avoid reconnection if server supports it.
-		  if(keep_alive && (keep_alive_max>1 || keep_alive_max==-1))
+		  if(keep_alive && (keep_alive_max>1 || keep_alive_max==-1) && use_head)
 		  {
 		     SendArrayInfoRequest();
 		  }
