@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.71 2003/02/26 10:00:02 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.73 2003/08/11 15:06:34 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -29,6 +29,7 @@
 #include "LsCache.h"
 #include "misc.h"
 #include "log.h"
+#include "DirColors.h"
 
 static bool token_eq(const char *buf,int len,const char *token)
 {
@@ -264,9 +265,9 @@ void file_info::clear()
    year=-1;
    month=-1;
    day=0;
-   hour=0;
-   minute=0;
-   second=0;
+   hour=-1;
+   minute=-1;
+   second=-1;
    month_name[0]=0;
    size_str[0]=0;
    perms[0]=0;
@@ -291,7 +292,7 @@ bool file_info::validate()
 	 year+=1900;
    }
 
-   if(day<1 || day>31 || hour<0 || hour>23 || minute<0 || minute>59
+   if(day<1 || day>31 || hour<-1 || hour>23 || minute<-1 || minute>59
    || (month==-1 && !is_ascii_alnum(month_name[0])))
       return false;
 
@@ -536,12 +537,109 @@ static bool try_squid_ftp(file_info &info,const char *str,char *str_with_tags)
    return true;
 }
 
+static bool try_wwwoffle_ftp(file_info &info,const char *buf,
+   const char *ext,const char **info_string,int *info_string_len)
+{
+   info.clear();
+
+   //   Perms Nlnk user [group] size Mon DD (YYYY or hh:mm)
+   int perms_code;
+   char year_or_time[6];
+   int consumed;
+
+   int n=sscanf(buf,"%11s %d %31s %31s %lld %3s %2d %5s%n",info.perms,&info.nlink,
+	       info.user,info.group,&info.size,info.month_name,&info.day,
+	       year_or_time,&consumed);
+   if(n==4) // bsd-like listing without group?
+   {
+      info.group[0]=0;
+      n=sscanf(buf,"%11s %d %31s %lld %3s %2d %5s%n",info.perms,&info.nlink,
+	    info.user,&info.size,info.month_name,&info.day,year_or_time,&consumed);
+   }
+   if(n>=7 && -1!=(perms_code=parse_perms(info.perms+1))
+   && -1!=(info.month=parse_month(info.month_name))
+   && -1!=parse_year_or_time(year_or_time,&info.year,&info.hour,&info.minute))
+   {
+      sprintf(info.size_str,"%lld",info.size);
+      if(info.perms[0]=='d')
+	 info.is_directory=true;
+      else if(info.perms[0]=='l')
+      {
+	 info.is_sym_link=true;
+	 char *p=strstr(ext,"-&gt; ");
+	 if(p)
+	 {
+	    info.sym_link=xstrdup(p+6);
+	    info.free_sym_link=true;
+	 }
+      }
+      *info_string=buf;
+      *info_string_len=consumed;
+      debug("wwwoffle ftp over http proxy listing matched");
+      return true;
+   }
+   return false;
+}
+
+//     4096             Jun 25 23:48Directory| ***
+//     4096             Jun 23  2002Directory| ***
+//       50             Jul  9 18:37Symbolic Link| ***
+//      217             Jun  3 06:01Plain Text| ***
+//    40419             Jul  6 13:06Hypertext Markup Language| ***
+// 14289850             Jul 16 17:04Windows Bitmap| ***
+//  6668926             Jul 17 16:01Binary Executable| ***
+static bool try_csm_proxy(file_info &info,const char *str)
+{
+   info.clear();
+
+   int n;
+   int status = false;
+   char additional_file_info[32];
+   int has_additional_file_info = false;
+
+   memset(additional_file_info, '\0', sizeof (additional_file_info));
+
+   // try to match hour:minute
+   if (5 <= (n = sscanf(str,"%d %3s %d %2d:%2d%32c",
+	       &info.size, info.month_name, &info.day, &info.hour, &info.minute, additional_file_info))) {
+      status = true;
+      if (6 == n)
+	  has_additional_file_info = true;
+   } else {
+       // try to match year instead of hour:minute
+       info.clear();
+       if (4 <= (n = sscanf(str,"%d %3s %d %4d%32c",
+		       &info.size, info.month_name, &info.day, &info.year, additional_file_info))) {
+	   status = true;
+	   if (5 == n)
+	       has_additional_file_info = true;
+       }
+   }
+
+   if (true == status) {
+      debug("csm_proxy listing matched");
+      sprintf(info.size_str, "%ld", info.size);
+      if (true == has_additional_file_info && strlen(additional_file_info)) {
+	  if (!strcmp("Symbolic Link", additional_file_info)) {
+	      info.is_sym_link = true;
+	  } else if (!strcmp("Directory", additional_file_info)) {
+	      info.is_directory = true;
+	  } else {
+	      // fprintf(stderr, "try_csm_proxy: |%s|\n", additional_file_info);
+	      Log::global->Format(10,
+		      "* try_csm_proxy: unknown file type '%s'\n",
+		      additional_file_info);
+	  }
+      }
+   }
+   return status;
+}
 
 // this procedure is highly inefficient in some cases,
 // esp. when it has to return for more data many times.
 static int parse_html(const char *buf,int len,bool eof,Buffer *list,
       FileSet *set,FileSet *all_links,ParsedURL *prefix,char **base_href,
-      LsOptions *lsopt=0)
+      LsOptions *lsopt=0, int color = 0)
 {
    const char *end=buf+len;
    const char *less=find_char(buf,len,'<');
@@ -804,6 +902,11 @@ parse_url_again:
 	    strcpy(link_target,".");
 	    link_len=1;
 	 }
+	 if(link_target[0]=='.' && link_target[1]=='/')
+	 {
+	    link_len-=2;
+	    memmove(link_target,link_target+2,link_len+1);
+	 }
       }
       else
       {
@@ -839,10 +942,11 @@ parse_url_again:
    if(list && show_in_list)
    {
       const char *more1;
-      char *str,*str_with_tags;
+      char *str,*str_with_tags, *str2;
       char *line_add=(char*)alloca(link_len+128+2*1024);
       const char *info_string=0;
       int         info_string_len=0;
+      int type;
 
       if(!a_href)
 	 goto add_file_no_info;	// only <a href> tags can have useful info.
@@ -903,8 +1007,8 @@ parse_url_again:
 	 skip_len=eol-buf+eol_len;
 	 goto got_info;
       }
-      if(try_mini_proxy(info,buf)		&& info.validate()) goto got_info;
-      if(try_apache_unixlike(info,buf,more,more1,&info_string,&info_string_len)
+      if(try_mini_proxy(info,str)		&& info.validate()) goto got_info;
+      if(try_apache_unixlike(info,str,more,more1,&info_string,&info_string_len)
       && info.validate())
 	 goto got_info;
       if(try_roxen(info,str)			&& info.validate()) goto got_info;
@@ -914,6 +1018,18 @@ parse_url_again:
 	 skip_len=eol-buf+eol_len;
 	 goto got_info;
       }
+      // wwwoffle
+      str2=string_alloca(less-buf+1);
+      memcpy(str2,buf,less-buf);
+      str2[less-buf]=0;
+      if(try_wwwoffle_ftp(info,str2,str,&info_string,&info_string_len)
+      && info.validate())
+      {
+	 // skip rest of line, because there may be href to link target.
+	 skip_len=eol-buf+eol_len;
+	 goto got_info;
+      }
+      if(try_csm_proxy(info,str) && info.validate()) goto got_info;
 
    add_file_no_info:
       sprintf(line_add,"%s  --  %s",
@@ -943,9 +1059,50 @@ parse_url_again:
 	 else
 	    strcpy(info.perms,"-rw-r--r--");
       }
-      sprintf(line_add,"%s  %11s  %04d-%s-%02d %02d:%02d  %s",
-	 info.perms,info.size_str,info.year,info.month_name,info.day,
-	 info.hour,info.minute,link_target);
+
+      sprintf(line_add,"%s  %11s  %04d-%s-%02d",
+	 info.perms,info.size_str,info.year,info.month_name,info.day);
+
+      if (info.hour >= 0 || info.minute >= 0) {
+	  if (info.hour >= 0) {
+	      char hour[6];
+	      sprintf(hour, " %02d:", info.hour);
+	      strcat(line_add, hour);
+	  } else {
+	      strcat(line_add, " --:");
+	  }
+
+	  if (info.minute >= 0) {
+	      char minute[4];
+	      sprintf(minute, "%02d", info.minute);
+	      strcat(line_add, minute);
+	  } else {
+	      strcat(line_add, "--");
+	  }
+      } else {
+	  // neither hour nor minute are given
+	  strcat(line_add, "      ");
+      }
+
+      strcat(line_add, "  ");
+
+      type = FileInfo::NORMAL;
+
+      if (color) {
+	  if(info.is_directory)
+	      type = FileInfo::DIRECTORY;
+	  else if(info.is_sym_link && !info.sym_link)
+	      type = FileInfo::SYMLINK;
+      }
+
+      if (color && FileInfo::NORMAL != type && all_links && !all_links->FindByName(link_target)) {
+	  list->Put(line_add);
+	  DirColors::GetInstance()->PutColored(list, link_target, type);
+	  line_add[0] = '\0'; // reset
+      } else {
+	  strcat(line_add, link_target);
+      }
+
    append_symlink_maybe:
       if(info.sym_link)
 	 sprintf(line_add+strlen(line_add)," -> %s",info.sym_link);
@@ -1082,7 +1239,7 @@ int HttpDirList::Do()
 
    int m=STALL;
 
-   int n=parse_html(b,len,ubuf->Eof(),buf,0,&all_links,curr_url,&base_href,&ls_options);
+   int n=parse_html(b,len,ubuf->Eof(),buf,0,&all_links,curr_url,&base_href,&ls_options, color);
    if(n>0)
    {
       ubuf->Skip(n);
