@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.96 2003/02/14 09:32:04 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.99 2003/04/30 12:26:32 lav Exp $ */
 
 #include <config.h>
 
@@ -725,19 +725,33 @@ int   MirrorJob::Do()
 	 if(!file)
 	    goto pre_TARGET_CHMOD;
 	 to_rm->next();
-	 ArgV *args=new ArgV("rm");
-	 args->Append(file->name);
-	 args->seek(1);
-	 rmJob *j=new rmJob(target_session->Clone(),args);
-	 j->SetParentFg(this);
-	 j->cmdline=args->Combine();
-	 AddWaiting(j);
-	 transfer_count++;
+	 if(script)
+	 {
+	    ArgV args("rm");
+	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
+	       args.Append("-r");
+	    args.Append(target_session->GetFileURL(file->name));
+	    char *cmd=args.CombineQuoted();
+	    fprintf(script,"%s\n",cmd);
+	    xfree(cmd);
+	 }
+	 if(!script_only)
+	 {
+	    ArgV *args=new ArgV("rm");
+	    args->Append(file->name);
+	    args->seek(1);
+	    rmJob *j=new rmJob(target_session->Clone(),args);
+	    j->SetParentFg(this);
+	    j->cmdline=args->Combine();
+	    AddWaiting(j);
+	    transfer_count++;
+	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
+	       j->Recurse();
+	 }
 	 if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
 	 {
 	    Report(_("Removing old directory `%s'"),
 		     dir_file(target_relative_dir,file->name));
-	    j->Recurse();
 	 }
 	 else
 	 {
@@ -769,23 +783,50 @@ int   MirrorJob::Do()
 	 if(!file)
 	    goto pre_FINISHING;
 	 to_transfer->next();
+	 if((file->defined&file->TYPE) && file->filetype==file->SYMLINK)
+	    continue;
 	 if(!(file->defined&file->MODE))
 	    continue;
-	 ArgV *a=new ArgV("chmod");
-	 a->Append(file->name);
-	 a->seek(1);
-	 ChmodJob *cj=new ChmodJob(target_session->Clone(),file->mode&~get_mode_mask(),a);
-	 AddWaiting(cj);
-	 transfer_count++;
-	 cj->SetParentFg(this);
-	 cj->cmdline=a->Combine();
-	 cj->BeQuiet();   // chmod is not supported on all servers; be quiet.
-	 m=MOVED;
+	 mode_t mode_mask=get_mode_mask();
+	 mode_t def_mode=(file->filetype==file->DIRECTORY?0775:0664)&~mode_mask;
+	 if(target_is_local && file->mode==def_mode)
+	 {
+	    struct stat st;
+	    if(!target_is_local || lstat(dir_file(target_dir,file->name),&st)==-1)
+	       continue;
+	    if((st.st_mode&07777)==(file->mode&~mode_mask))
+	       continue;
+	 }
+	 if(script)
+	 {
+	    ArgV args("chmod");
+	    char m[16];
+	    sprintf(m,"%03lo",(unsigned long)(file->mode&~mode_mask));
+	    args.Append(m);
+	    args.Append(target_session->GetFileURL(file->name));
+	    char *cmd=args.CombineQuoted();
+	    fprintf(script,"%s\n",cmd);
+	    xfree(cmd);
+	 }
+	 if(!script_only)
+	 {
+	    ArgV *a=new ArgV("chmod");
+	    a->Append(file->name);
+	    a->seek(1);
+	    ChmodJob *cj=new ChmodJob(target_session->Clone(),
+				 file->mode&~mode_mask,a);
+	    AddWaiting(cj);
+	    transfer_count++;
+	    cj->SetParentFg(this);
+	    cj->cmdline=a->Combine();
+	    cj->BeQuiet(); // chmod is not supported on all servers; be quiet.
+	    m=MOVED;
+	 }
       }
       break;
 
    pre_FINISHING:
-      if(target_is_local)     // FIXME
+      if(target_is_local && !script_only)     // FIXME
       {
 	 to_transfer->LocalUtime(target_dir,/*only_dirs=*/true);
 	 if(flags&ALLOW_CHOWN)
@@ -956,9 +997,14 @@ mode_t MirrorJob::get_mode_mask()
       mode_mask|=S_ISUID|S_ISGID;
    if(!(flags&NO_UMASK))
    {
-      mode_t u=umask(022); // get+set
-      umask(u);	    // retore
-      mode_mask|=u;
+      if(target_is_local)
+      {
+	 mode_t u=umask(022); // get+set
+	 umask(u);	      // retore
+	 mode_mask|=u;
+      }
+      else
+	 mode_mask|=022;   // sane default.
    }
    return mode_mask;
 }
@@ -1043,7 +1089,9 @@ CMD(mirror)
       {"parallel",optional_argument,0,'P'},
       {"ignore-time",no_argument,0,256+'i'},
       {"log",required_argument,0,256+'s'},
-      {"script",required_argument,0,256+'S'},
+      {"script",    required_argument,0,256+'S'},
+      {"just-print",optional_argument,0,256+'S'},
+      {"dry-run",   optional_argument,0,256+'S'},
       {0}
    };
 
@@ -1175,6 +1223,8 @@ CMD(mirror)
 	 script_only=true;
       case(256+'s'):
 	 script_file=optarg;
+	 if(script_file==0)
+	    script_file="-";
 	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
@@ -1281,8 +1331,8 @@ CMD(mirror)
       j->RemoveSourceFiles();
    if(parallel<0)
       parallel=0;
-   if(parallel>16)
-      parallel=16;   // a sane limit.
+   if(parallel>64)
+      parallel=64;   // a (in)sane limit.
    if(parallel)
       j->SetParallel(parallel);
 
