@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.28 2001/12/20 11:14:44 lav Exp $ */
+/* $Id: NetAccess.h,v 1.30 2002/12/25 13:44:31 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -69,6 +69,8 @@ protected:
    static int SocketPort(const sockaddr_u *u);
    static socklen_t SocketAddrLen(const sockaddr_u *u);
    static int SocketConnect(int fd,const sockaddr_u *u);
+   int SocketCreate(int,int,int);
+   int SocketCreateTCP(int);
 
    char	 *proxy;
    char	 *proxy_port;
@@ -129,15 +131,21 @@ private:
    static int total_xfer_number;
    static bool total_reconfig_needed;
    static void ReconfigTotal();
-   static BytesPool total;
-   BytesPool one;
+   static BytesPool total[2];
+   BytesPool one[2];
 
 public:
    RateLimit(const char *closure);
    ~RateLimit();
 
-   int BytesAllowed();
-   void BytesUsed(int);
+   enum dir_t { GET=0, PUT=1 };
+
+   int BytesAllowed(dir_t how);
+   int BytesAllowedToGet() { return BytesAllowed(GET); }
+   int BytesAllowedToPut() { return BytesAllowed(PUT); }
+   void BytesUsed(int,dir_t);
+   void BytesGot(int b) { BytesUsed(b,GET); }
+   void BytesPut(int b) { BytesUsed(b,PUT); }
 
    void Reconfig(const char *name,const char *c);
 };
