@@ -18,19 +18,21 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.23 2002/06/18 16:51:59 lav Exp $ */
+/* $Id: buffer.h,v 1.25 2003/06/21 08:40:08 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
 
 #include "SMTask.h"
 #include "Filter.h"
+#include "Timer.h"
 #include "fg.h"
 #include "xstring.h"
 
 #include <stdarg.h>
 
-#define GET_BUFSIZE 0x4000
+#define GET_BUFSIZE 0x10000
+#define PUT_LL_MIN  0x2000
 
 class Speedometer;
 
@@ -138,15 +140,17 @@ public:
 class IOBufferFDStream : public IOBuffer
 {
    FDStream *stream;
+   class Timer *put_ll_timer;
 
    int Get_LL(int size);
    int Put_LL(const char *buf,int size);
 
 protected:
-   ~IOBufferFDStream() { delete stream; }
+   ~IOBufferFDStream() { delete stream; delete put_ll_timer; }
 
 public:
-   IOBufferFDStream(FDStream *o,dir_t m) : IOBuffer(m) { stream=o; }
+   IOBufferFDStream(FDStream *o,dir_t m,Timer *t=0)
+      : IOBuffer(m) { stream=o; put_ll_timer=t; }
    int Do();
    bool Done();
    FgData *GetFgData(bool fg);
