@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.189 2003/01/27 15:52:46 lav Exp $ */
+/* $Id: commands.cc,v 1.193 2003/06/16 11:32:51 lav Exp $ */
 
 #include <config.h>
 
@@ -94,7 +94,7 @@ CMD(ver);   CMD(close);  CMD(bookmark);CMD(lftp);
 CMD(echo);  CMD(suspend);CMD(sleep);   CMD(slot);
 CMD(at);    CMD(find);   CMD(command); CMD(module);
 CMD(lpwd);  CMD(glob);	 CMD(chmod);   CMD(queue);
-CMD(repeat);CMD(get1);   CMD(history);
+CMD(repeat);CMD(get1);   CMD(history); CMD(tasks);
 
 #ifdef MODULE_CMD_MIRROR
 # define cmd_mirror 0
@@ -469,6 +469,8 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"bzmore",   cmd_cat,    0,
 	 N_("Same as more, but filter each file through bzcat\n")},
 
+   {".tasks",  cmd_tasks,  0,0},
+
    {NULL,NULL}
 };
 
@@ -701,6 +703,7 @@ Job *CmdExec::builtin_lftp()
       /* if no lftp-specific options were found, call open */
       return builtin_open();
    }
+   exit_code=0;
    return 0;
 }
 
@@ -938,24 +941,21 @@ Job *CmdExec::builtin_restart()
 Job *CmdExec::builtin_glob()
 {
    const char *op=args->a0();
-   bool files_only=true;
-   bool dirs_only=false;
    int opt;
+   GlobURL::type_select glob_type=GlobURL::FILES_ONLY;
 
    while((opt=args->getopt("+adf"))!=EOF)
    {
       switch(opt)
       {
       case('a'):
-	 files_only=dirs_only=false;
+	 glob_type=GlobURL::ALL;
 	 break;
       case('d'):
-	 dirs_only=true;
-	 files_only=false;
+	 glob_type=GlobURL::DIRS_ONLY;
 	 break;
       case('f'):
-	 files_only=true;
-	 dirs_only=false;
+	 glob_type=GlobURL::FILES_ONLY;
 	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),op);
@@ -981,11 +981,7 @@ Job *CmdExec::builtin_glob()
       args->rewind();
       return cmd_command(this);
    }
-   glob=new GlobURL(session,pat);
-   if(dirs_only)
-      glob->glob->DirectoriesOnly();
-   if(files_only)
-      glob->glob->FilesOnly();
+   glob=new GlobURL(session,pat,glob_type);
    builtin=BUILTIN_GLOB;
    return this;
 }
@@ -2359,6 +2355,7 @@ CMD(help)
 
    parent->print_cmd_index();
 
+   exit_code=0;
    return 0;
 }
 
@@ -3071,3 +3068,10 @@ CMD(slot)
       return j;
    }
 }
+
+CMD(tasks)
+{
+   printf("task_count=%d\n",SMTask::TaskCount());
+   exit_code=0;
+   return 0;
+}
