@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.63 2002/07/24 11:09:56 lav Exp $ */
+/* $Id: complete.cc,v 1.65 2003/05/23 07:07:44 lav Exp $ */
 
 #include <config.h>
 
@@ -86,6 +86,8 @@ char *command_generator(const char *text,int state)
    while ((name=CmdExec::CmdByIndex(cindex)->name)!=0)
    {
       cindex++;
+      if(name[0]=='.' && len==0)
+	 continue;   // skip hidden commands
       if(strncasecmp(name,text,len)==0)
 	 return(xstrdup(name));
    }
@@ -676,7 +678,8 @@ static char **lftp_completion (const char *text,int start,int end)
 
       SignalHook::ResetCount(SIGINT);
       glob_res=NULL;
-      rg=new GlobURL(completion_shell->session,pat);
+      rg=new GlobURL(completion_shell->session,pat,
+		     type==REMOTE_DIR?GlobURL::DIRS_ONLY:GlobURL::ALL);
 
       rl_save_prompt();
 
@@ -692,8 +695,6 @@ static char **lftp_completion (const char *text,int start,int end)
 	    rg->glob->CaseFold();
 	 } else
 	    rl_variable_bind("completion-ignore-case", "0");
-	 if(type==REMOTE_DIR)
-	    rg->glob->DirectoriesOnly();
 
 	 Timer status_timer;
 	 status_timer.SetMilliSeconds(20);
