@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.cc,v 1.257 2003/02/28 07:12:36 lav Exp $ */
+/* $Id: ftpclass.cc,v 1.264 2003/08/21 08:43:24 lav Exp $ */
 
 #include <config.h>
 
@@ -814,18 +814,17 @@ void Ftp::CatchSIZE(int act)
    if(!array_for_info)
       return;
 
+   long long size=NO_SIZE;
+
    if(is2XX(act))
    {
       if(strlen(line)>4 && is_ascii_digit(line[4]))
-	 array_for_info[array_ptr].size=atol(line+4);
-      else
-	 array_for_info[array_ptr].size=-1;
+	 sscanf(line+4,"%lld",&size);
    }
    else	if(is5XX(act))
    {
       if(act==500 || act==502)
 	 size_supported=false;
-      array_for_info[array_ptr].size=-1;
    }
    else
    {
@@ -833,6 +832,11 @@ void Ftp::CatchSIZE(int act)
       return;
    }
 
+   if(size<1)
+      size=NO_SIZE;
+
+   array_for_info[array_ptr].size=size;
+
    array_for_info[array_ptr].get_size=false;
    if(!array_for_info[array_ptr].get_time)
       array_ptr++;
@@ -842,16 +846,23 @@ void Ftp::CatchSIZE(int act)
 }
 void Ftp::CatchSIZE_opt(int act)
 {
+   long long size=NO_SIZE;
+
    if(is2XX(act) && strlen(line)>4 && is_ascii_digit(line[4]))
    {
-      entity_size=atol(line+4);
+      sscanf(line+4,"%lld",&size);
    }
    else
    {
       if(act==500 || act==502)
 	 size_supported=false;
-      entity_size=NO_SIZE;
    }
+
+   if(size<1)
+      return;
+
+   entity_size=size;
+
    if(opt_size)
    {
       *opt_size=entity_size;
@@ -909,6 +920,10 @@ void Ftp::InitFtp()
    use_stat=true;
    stat_interval=1;
 
+   use_mdtm=true;
+   use_size=true;
+   use_telnet_iac=true;
+
    dos_path=false;
    vms_path=false;
    mdtm_supported=true;
@@ -1030,7 +1045,7 @@ bool Ftp::GetBetterConnection(int level,bool limit_reached)
    {
       Ftp *o=(Ftp*)fo; // we are sure it is Ftp.
 
-      if(o->IsConnected()<2)
+      if(o->GetConnectLevel()!=CL_LOGGED_IN)
 	 continue;
       if(!SameConnection(o))
 	 continue;
@@ -1647,12 +1662,12 @@ int   Ftp::Do()
 	 AddResp(RESP_TYPE_OK);
       }
 
-      if(opt_size && size_supported && file[0])
+      if(opt_size && size_supported && file[0] && use_size)
       {
 	 SendCmd2("SIZE",file);
 	 AddResp(RESP_RESULT_HERE,CHECK_SIZE_OPT);
       }
-      if(opt_date && mdtm_supported && file[0])
+      if(opt_date && mdtm_supported && file[0] && use_mdtm)
       {
 	 SendCmd2("MDTM",file);
 	 AddResp(RESP_RESULT_HERE,CHECK_MDTM_OPT);
@@ -2028,9 +2043,8 @@ int   Ftp::Do()
 	    TimeoutS(stat_time+stat_interval-now);
       }
 
-      // store mode is special - the data can be buffered
-      // so is COPY_* - no data connection at all.
-      if(mode==STORE || copy_mode!=COPY_NONE)
+      // FXP is special - no data connection at all.
+      if(copy_mode!=COPY_NONE)
 	 goto notimeout_return;
 
       goto usual_return;
@@ -2154,7 +2168,7 @@ void Ftp::SendArrayInfoRequests()
    for(int i=array_ptr; i<array_cnt; i++)
    {
       bool sent=false;
-      if(array_for_info[i].get_time && mdtm_supported)
+      if(array_for_info[i].get_time && mdtm_supported && use_mdtm)
       {
 	 SendCmd2("MDTM",ExpandTildeStatic(array_for_info[i].file));
 	 AddResp(RESP_RESULT_HERE,CHECK_MDTM);
@@ -2164,7 +2178,7 @@ void Ftp::SendArrayInfoRequests()
       {
 	 array_for_info[i].time=NO_DATE;
       }
-      if(array_for_info[i].get_size && size_supported)
+      if(array_for_info[i].get_size && size_supported && use_size)
       {
 	 SendCmd2("SIZE",ExpandTildeStatic(array_for_info[i].file));
 	 AddResp(RESP_RESULT_HERE,CHECK_SIZE);
@@ -2172,7 +2186,7 @@ void Ftp::SendArrayInfoRequests()
       }
       else
       {
-	 array_for_info[i].size=-1;
+	 array_for_info[i].size=NO_SIZE;
       }
       if(!sent)
       {
@@ -2421,6 +2435,12 @@ int  Ftp::ReceiveResp()
 
 void Ftp::SendUrgentCmd(const char *cmd)
 {
+   if(!use_telnet_iac)
+   {
+      SendCmd(cmd);
+      return;
+   }
+
    static const char pre_cmd[]={TELNET_IAC,TELNET_IP,TELNET_IAC,TELNET_SYNCH};
 
    int fl=fcntl(control_sock,F_GETFL);
@@ -2555,6 +2575,7 @@ void  Ftp::Disconnect()
 
    bool no_greeting=(!RespQueueIsEmpty() && RespQueue[RQ_head].check_case==CHECK_READY);
 
+   CloseRespQueue();
    DataAbort();
    DataClose();
    if(control_sock>=0 && state!=CONNECTING_STATE && !quit_sent
@@ -2779,7 +2800,7 @@ void  Ftp::SendCmd(const char *cmd,int len)
 	 cmd--;
 	 len++;
       }
-      else if(ch=='\377') // double chr(255) as in telnet protocol
+      else if(ch=='\377' && use_telnet_iac) // double chr(255) as in telnet protocol
       	 send_cmd_ptr[send_cmd_count++]='\377';
       send_cmd_ptr[send_cmd_count++]=prev_ch=ch;
       if(len==0 && ch!='\n')
@@ -3359,7 +3380,7 @@ void Ftp::CheckResp(int act)
       stat_time=now+2;
    }
 
-   if(act==150 && mode==RETRIEVE && opt_size && *opt_size==-1)
+   if(act==150 && mode==RETRIEVE && opt_size && *opt_size<0)
    {
       // try to catch size
       char *s=strrchr(line,'(');
@@ -3892,6 +3913,11 @@ void Ftp::Reconfig(const char *name)
    use_stat = QueryBool("use-stat",c);
    stat_interval = Query("stat-interval",c);
 
+   use_mdtm = QueryBool("use-mdtm",c);
+   use_size = QueryBool("use-size",c);
+
+   use_telnet_iac = QueryBool("use-telnet-iac",c);
+
    xfree(list_options);
    list_options = xstrdup(Query("list-options",c));
 
