@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.89 2002/01/21 16:25:17 lav Exp $ */
+/* $Id: ftpclass.h,v 1.92 2003/07/15 13:36:21 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -301,6 +301,11 @@ private:
    bool copy_allow_store;
    bool copy_failed;
 
+   bool use_mdtm;
+   bool use_size;
+
+   bool use_telnet_iac;
+
    bool use_stat;
    int  stat_interval;
 
@@ -330,13 +335,32 @@ public:
 
    void	 ResetLocationData();
 
-   int IsConnected()
+   enum ConnectLevel
+   {
+      CL_NOT_CONNECTED,
+      CL_CONNECTING,
+      CL_JUST_CONNECTED,
+      CL_NOT_LOGGED_IN,
+      CL_LOGGED_IN,
+      CL_JUST_BEFORE_DISCONNECT
+   };
+   ConnectLevel GetConnectLevel()
    {
       if(control_sock==-1)
-	 return 0;
-      if(state==CONNECTING_STATE || quit_sent)
-	 return 1;
-      return 2;
+	 return CL_NOT_CONNECTED;
+      if(state==CONNECTING_STATE)
+	 return CL_CONNECTING;
+      if(state==CONNECTED_STATE)
+	 return CL_JUST_CONNECTED;
+      if(state==USER_RESP_WAITING_STATE)
+	 return CL_NOT_LOGGED_IN;
+      if(quit_sent)
+	 return CL_JUST_BEFORE_DISCONNECT;
+      return CL_LOGGED_IN;
+   }
+   int IsConnected()
+   {
+      return GetConnectLevel()!=CL_NOT_CONNECTED;
    }
 
    int   Read(void *buf,int size);
