@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.82 2003/01/10 06:03:52 lav Exp $ */
+/* $Id: resource.cc,v 1.84 2003/07/15 13:36:21 lav Exp $ */
 
 #include <config.h>
 
@@ -162,7 +162,8 @@ const char *AuthArgValidate(char **s)
 #endif
 
 // Static array of objects is wrongly initialized by IRIX CC and Unixware c++.
-// So here goes list of arbitrarily named objects, they are not refered by name.
+// So here goes list of arbitrarily named objects, they are never referred
+// by name.
 static ResDecl
    ResDecl00a("ftp:acct",		  "",	   0,0),
    ResDecl00 ("ftp:anon-pass",		  "lftp@", 0,0),
@@ -197,9 +198,12 @@ static ResDecl
    ResDecl13 ("ftp:sync-mode",		  "on",    ResMgr::BoolValidate,0),
    ResDecl14 ("ftp:use-abor",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl15 ("ftp:use-fxp",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl15b("ftp:use-mdtm",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl15a("ftp:use-site-chmod",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl16a("ftp:use-site-idle",	  "no",    ResMgr::BoolValidate,0),
+   ResDecl16b("ftp:use-size",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl16 ("ftp:use-stat",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl16c("ftp:use-telnet-iac",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl17 ("ftp:use-quit",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl18 ("ftp:verify-address",	  "no",    ResMgr::BoolValidate,0),
    ResDecl19 ("ftp:verify-port",	  "no",    ResMgr::BoolValidate,0),
